@@ -36,6 +36,9 @@
 #include <dirent.h>
 #include <sys/vfs.h>
 #include <sys/utsname.h>
+#ifdef WITH_SYSTEMD
+#include <systemd/sd-daemon.h>
+#endif
 
 #include "automount.h"
 #if defined(LIBXML2_WORKAROUND) || defined(TIRPC_WORKAROUND)
@@ -51,6 +54,8 @@ const char *libdir = AUTOFS_LIB_DIR;	/* Location of library modules */
 const char *mapdir = AUTOFS_MAP_DIR;	/* Location of mount maps */
 const char *confdir = AUTOFS_CONF_DIR;	/* Location of autofs config file */
 
+unsigned int mp_mode = 0755;
+
 unsigned int nfs_mount_uses_string_options = 0;
 static struct nfs_mount_vers vers, check = {1, 1, 1};
 
@@ -65,7 +70,7 @@ unsigned int global_selection_options;
 long global_negative_timeout = -1;
 int do_force_unlink = 0;		/* Forceably unlink mount tree at startup */
 
-static int start_pipefd[2];
+static int start_pipefd[2] = {-1, -1};
 static int st_stat = 1;
 static int *pst_stat = &st_stat;
 static pthread_t state_mach_thid;
@@ -75,9 +80,6 @@ static sigset_t block_sigs;
 /* Pre-calculated kernel packet length */
 static size_t kpkt_len;
 
-/* Does kernel know about SOCK_CLOEXEC and friends */
-static int cloexec_works = 0;
-
 /* Attributes for creating detached and joinable threads */
 pthread_attr_t th_attr;
 pthread_attr_t th_attr_detached;
@@ -610,16 +612,7 @@ static int umount_subtree_mounts(struct autofs_point *ap, const char *path, unsi
 		}
 		list_del(&entry->entries);
 		mounts_mutex_unlock(ap);
-		if (ext_mount_remove(&entry->ext_mount, entry->fs)) {
-			if (umount_ent(ap, entry->fs))
-				debug(ap->logopt,
-				      "failed to umount external mount %s",
-				      entry->fs);
-			else
-				debug(ap->logopt,
-				      "umounted external mount %s",
-				      entry->fs);
-		}
+		umount_amd_ext_mount(ap, entry);
 		free_amd_entry(entry);
 	}
 done:
@@ -636,10 +629,6 @@ int umount_multi(struct autofs_point *ap, const char *path, int incl)
 
 	debug(ap->logopt, "path %s incl %d", path, incl);
 
-	/* If path is a mount it can't be a symlink */
-	if (is_mounted(_PATH_MOUNTED, path, MNTS_ALL))
-		goto real_mount;
-
 	if (lstat(path, &st)) {
 		warn(ap->logopt,
 		     "failed to stat directory or symlink %s", path);
@@ -670,21 +659,11 @@ int umount_multi(struct autofs_point *ap, const char *path, int incl)
 		}
 		list_del(&entry->entries);
 		mounts_mutex_unlock(ap);
-		if (ext_mount_remove(&entry->ext_mount, entry->fs)) {
-			if (umount_ent(ap, entry->fs))
-				debug(ap->logopt,
-				      "failed to umount external mount %s",
-				      entry->fs);
-			else
-				debug(ap->logopt,
-				      "umounted external mount %s",
-				      entry->fs);
-		}
+		umount_amd_ext_mount(ap, entry);
 		free_amd_entry(entry);
 		return 0;
 	}
 
-real_mount:
 	is_autofs_fs = 0;
 	if (master_find_submount(ap, path))
 		is_autofs_fs = 1;
@@ -1000,7 +979,7 @@ static int set_log_priority(const char *path, int priority)
 	 * Specify O_NONBLOCK so that the open will fail if there is no
 	 * daemon reading from the other side of the FIFO.
 	 */
-	fd = open(fifo_name, O_WRONLY|O_NONBLOCK);
+	fd = open_fd(fifo_name, O_WRONLY|O_NONBLOCK);
 	if (fd < 0) {
 		fprintf(stderr, "%s: open of %s failed with %s\n",
 			__FUNCTION__, fifo_name, strerror(errno));
@@ -1216,7 +1195,7 @@ static int handle_packet(struct autofs_point *ap)
 	return -1;
 }
 
-static void become_daemon(unsigned foreground, unsigned daemon_check)
+static void become_daemon(unsigned int flags)
 {
 	FILE *pidfp;
 	char buf[MAX_ERR_BUF];
@@ -1230,21 +1209,31 @@ static void become_daemon(unsigned foreground, unsigned daemon_check)
 		exit(0);
 	}
 
-	if (open_pipe(start_pipefd) < 0) {
-		fprintf(stderr, "%s: failed to create start_pipefd.\n",
-			program);
-		exit(0);
-	}
-
 	/* Detach from foreground process */
-	if (foreground) {
-		if (daemon_check && !aquire_flag_file()) {
+	if (flags & DAEMON_FLAGS_FOREGROUND &&
+	   !(flags & DAEMON_FLAGS_SYSTEMD_SERVICE)) {
+		if ((flags & DAEMON_FLAGS_CHECK_DAEMON) && !aquire_flag_file()) {
 			fprintf(stderr, "%s: program is already running.\n",
 				program);
 			exit(1);
 		}
 		log_to_stderr();
+	} else if (flags & DAEMON_FLAGS_SYSTEMD_SERVICE) {
+		if ((flags & DAEMON_FLAGS_CHECK_DAEMON) && !aquire_flag_file()) {
+			fprintf(stderr, "%s: program is already running.\n",
+				program);
+			exit(1);
+		}
+		open_log();
 	} else {
+		int nullfd;
+
+		if (open_pipe(start_pipefd) < 0) {
+			fprintf(stderr, "%s: failed to create start_pipefd.\n",
+				program);
+			exit(0);
+		}
+
 		pid = fork();
 		if (pid > 0) {
 			close(start_pipefd[1]);
@@ -1259,7 +1248,7 @@ static void become_daemon(unsigned foreground, unsigned daemon_check)
 		}
 		close(start_pipefd[0]);
 
-		if (daemon_check && !aquire_flag_file()) {
+		if ((flags & DAEMON_FLAGS_CHECK_DAEMON) && !aquire_flag_file()) {
 			fprintf(stderr, "%s: program is already running.\n",
 				program);
 			/* Return success if already running */
@@ -1282,7 +1271,30 @@ static void become_daemon(unsigned foreground, unsigned daemon_check)
 			close(start_pipefd[1]);
 			exit(*pst_stat);
 		}
-		log_to_syslog();
+
+		/* Redirect all our file descriptors to /dev/null */
+		nullfd = open("/dev/null", O_RDWR);
+		if (nullfd < 0) {
+			char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
+			fprintf(stderr, "cannot open /dev/null: %s", estr);
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+			exit(*pst_stat);
+		}
+
+		if (dup2(nullfd, STDIN_FILENO) < 0 ||
+		    dup2(nullfd, STDOUT_FILENO) < 0 ||
+		    dup2(nullfd, STDERR_FILENO) < 0) {
+			char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
+			fprintf(stderr,
+				"redirecting file descriptors failed: %s", estr);
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+			exit(*pst_stat);
+		}
+
+		open_log();
+		close(nullfd);
 	}
 
 	/* Write pid file if requested */
@@ -1914,9 +1926,15 @@ static void usage(void)
 		"	-h --help	this text\n"
 		"	-p --pid-file f write process id to file f\n"
 		"	-t --timeout n	auto-unmount in n seconds (0-disable)\n"
+		"	-M --master-wait n\n"
+	        "			maximum wait time (seconds) for master\n"
+	        "			map to become available\n"
 		"	-v --verbose	be verbose\n"
 		"	-d --debug	log debuging info\n"
-		"	-D --define	define global macro variable\n"
+		"	-Dvariable=value, --define variable=value\n"
+		"			define global macro variable\n"
+		"	-S --systemd-service\n"
+		"			run automounter as a systemd service\n"
 		"	-f --foreground do not fork into background\n"
 		"	-r --random-multimount-selection\n"
 		"			use ramdom replicated server selection\n"
@@ -1968,6 +1986,15 @@ static void show_build_info(void)
 	}
 #endif
 
+#ifdef WITH_SYSTEMD
+	printf("WITH_SYSTEMD ");
+	count = count + 13;
+
+	if (count > 60) {
+		printf("\n  ");
+		count = 0;
+	}
+#endif
 
 #ifdef WITH_HESIOD
 	printf("WITH_HESIOD ");
@@ -2092,6 +2119,31 @@ static void remove_empty_args(char **argv, int *argc)
 	*argc = j;
 }
 
+static void do_master_list_reset(struct master *master)
+{
+	struct list_head *head, *p, *n;
+
+	master_mutex_lock();
+
+	head = &master->mounts;
+	n = head->next;
+	while (n != head) {
+		struct master_mapent *entry;
+
+		p = n;
+		n = p->next;
+
+		entry = list_entry(p, struct master_mapent, list);
+
+		if (!list_empty(&entry->list))
+			list_del(&entry->list);
+		master_free_mapent_sources(entry, 1);
+		master_free_mapent(entry);
+	}
+
+	master_mutex_unlock();
+}
+
 static int do_master_read_master(struct master *master, int wait)
 {
 	sigset_t signalset;
@@ -2111,6 +2163,8 @@ static int do_master_read_master(struct master *master, int wait)
 	while (1) {
 		struct timespec t = { retry_wait, 0 };
 
+		do_master_list_reset(master);
+
 		age = monotonic_time(NULL);
 		if (master_read_master(master, age, 0)) {
 			ret = 1;
@@ -2139,14 +2193,14 @@ int main(int argc, char *argv[])
 {
 	int res, opt, status;
 	int logpri = -1;
-	unsigned ghost, logging, daemon_check;
-	unsigned dumpmaps, foreground, have_global_options;
+	unsigned int flags;
+	unsigned int logging;
 	unsigned master_read;
 	int master_wait;
 	time_t timeout;
 	time_t age = monotonic_time(NULL);
 	struct rlimit rlim;
-	const char *options = "+hp:t:vmdD:fVrO:l:n:CFM";
+	const char *options = "+hp:t:vmdD:SfVrO:l:n:CFM";
 	static const struct option long_options[] = {
 		{"help", 0, 0, 'h'},
 		{"pid-file", 1, 0, 'p'},
@@ -2154,6 +2208,7 @@ int main(int argc, char *argv[])
 		{"verbose", 0, 0, 'v'},
 		{"debug", 0, 0, 'd'},
 		{"define", 1, 0, 'D'},
+		{"systemd-service", 0, 0, 'S'},
 		{"foreground", 0, 0, 'f'},
 		{"random-multimount-selection", 0, 0, 'r'},
 		{"negative-timeout", 1, 0, 'n'},
@@ -2183,17 +2238,15 @@ int main(int argc, char *argv[])
 
 	nfs_mount_uses_string_options = check_nfs_mount_version(&vers, &check);
 
+	flags = defaults_get_browse_mode() ? DAEMON_FLAGS_GHOST : 0;
+	flags |= DAEMON_FLAGS_CHECK_DAEMON;
+
 	kpkt_len = get_kpkt_len();
 	master_wait = defaults_get_master_wait();
 	timeout = defaults_get_timeout();
-	ghost = defaults_get_browse_mode();
 	logging = defaults_get_logging();
 	global_selection_options = 0;
 	global_options = NULL;
-	have_global_options = 0;
-	foreground = 0;
-	dumpmaps = 0;
-	daemon_check = 1;
 
 	remove_empty_args(argv, &argc);
 
@@ -2224,8 +2277,12 @@ int main(int argc, char *argv[])
 			macro_parse_globalvar(optarg);
 			break;
 
+		case 'S':
+			flags |= DAEMON_FLAGS_SYSTEMD_SERVICE;
+			break;
+
 		case 'f':
-			foreground = 1;
+			flags |= DAEMON_FLAGS_FOREGROUND;
 			break;
 
 		case 'V':
@@ -2241,7 +2298,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'm':
-			dumpmaps = 1;
+			flags |= DAEMON_FLAGS_DUMP_MAPS;
 			break;
 
 		case 'M':
@@ -2249,9 +2306,9 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'O':
-			if (!have_global_options) {
+			if (!(flags & DAEMON_FLAGS_HAVE_GLOBAL_OPTIONS)) {
 				global_options = strdup(optarg);
-				have_global_options = 1;
+				flags |= DAEMON_FLAGS_HAVE_GLOBAL_OPTIONS;
 				break;
 			}
 			printf("%s: global options already specified.\n",
@@ -2276,7 +2333,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'C':
-			daemon_check = 0;
+			flags &= ~DAEMON_FLAGS_CHECK_DAEMON;
 			break;
 
 		case 'F':
@@ -2326,16 +2383,9 @@ int main(int argc, char *argv[])
 		exit(exit_code);
 	}
 
-#if 0
-	if (!load_autofs4_module()) {
-		fprintf(stderr, "%s: can't load %s filesystem module.\n",
-			program, FS_MODULE_NAME);
-		exit(1);
-	}
-#endif
-
 	/* Don't need the kernel module just to look at the configured maps */
-	if (!dumpmaps && (!query_kproto_ver() || get_kver_major() < 5)) {
+	if (!(flags & DAEMON_FLAGS_DUMP_MAPS) &&
+	   (!query_kproto_ver() || get_kver_major() < 5)) {
 		fprintf(stderr,
 			"%s: test mount forbidden or "
 			"incorrect kernel protocol version, "
@@ -2366,7 +2416,7 @@ int main(int argc, char *argv[])
 	/* Get processor information for predefined escapes */
 	macro_init();
 
-	if (dumpmaps) {
+	if (flags & DAEMON_FLAGS_DUMP_MAPS) {
 		struct master_mapent *entry;
 		struct list_head *head, *p;
 		struct mapent_cache *nc;
@@ -2384,9 +2434,9 @@ int main(int argc, char *argv[])
 		}
 
 		if (master)
-			master_list = master_new(NULL, timeout, ghost);
+			master_list = master_new(NULL, timeout, flags);
 		else
-			master_list = master_new(master, timeout, ghost);
+			master_list = master_new(master, timeout, flags);
 		if (!master_list) {
 			printf("%s: can't create master map", program);
 			macro_free_global_table();
@@ -2432,9 +2482,9 @@ int main(int argc, char *argv[])
 	}
 
 	if (argc == 0)
-		master_list = master_new(NULL, timeout, ghost);
+		master_list = master_new(NULL, timeout, flags);
 	else
-		master_list = master_new(argv[0], timeout, ghost);
+		master_list = master_new(argv[0], timeout, flags);
 
 	if (!master_list) {
 		printf("%s: can't create master map %s", program, argv[0]);
@@ -2442,13 +2492,15 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	become_daemon(foreground, daemon_check);
+	become_daemon(flags);
 
 	if (pthread_attr_init(&th_attr)) {
 		logerr("%s: failed to init thread attribute struct!",
 		     program);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2457,8 +2509,10 @@ int main(int argc, char *argv[])
 	if (pthread_attr_init(&th_attr_detached)) {
 		logerr("%s: failed to init thread attribute struct!",
 		     program);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2468,8 +2522,10 @@ int main(int argc, char *argv[])
 			&th_attr_detached, PTHREAD_CREATE_DETACHED)) {
 		logerr("%s: failed to set detached thread attribute!",
 		     program);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2480,8 +2536,10 @@ int main(int argc, char *argv[])
 			&th_attr_detached, detached_thread_stack_size)) {
 		logerr("%s: failed to set stack size thread attribute!",
 		       program);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2492,8 +2550,10 @@ int main(int argc, char *argv[])
 			&th_attr_detached, &detached_thread_stack_size)) {
 		logerr("%s: failed to get detached thread stack size!",
 		       program);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2510,8 +2570,10 @@ int main(int argc, char *argv[])
 		logerr("%s: failed to create thread data key for std env vars!",
 		       program);
 		master_kill(master_list);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2522,8 +2584,10 @@ int main(int argc, char *argv[])
 		logerr("%s: failed to create thread data key for attempt ID!",
 		       program);
 		master_kill(master_list);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2534,8 +2598,10 @@ int main(int argc, char *argv[])
 	if (!alarm_start_handler()) {
 		logerr("%s: failed to create alarm handler thread!", program);
 		master_kill(master_list);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2544,8 +2610,10 @@ int main(int argc, char *argv[])
 	if (!st_start_handler()) {
 		logerr("%s: failed to create FSM handler thread!", program);
 		master_kill(master_list);
-		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-		close(start_pipefd[1]);
+		if (start_pipefd[1] != -1) {
+			res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+			close(start_pipefd[1]);
+		}
 		release_flag_file();
 		macro_free_global_table();
 		exit(1);
@@ -2581,6 +2649,8 @@ int main(int argc, char *argv[])
 			 * Failed to read master map, continue with what
 			 * we have anyway.
 			 */
+			do_master_list_reset(master_list);
+			age = monotonic_time(NULL);
 			master_read_master(master_list, age, 1);
 		}
 	}
@@ -2591,9 +2661,16 @@ int main(int argc, char *argv[])
 	 */
 	do_force_unlink = 0;
 
-	st_stat = 0;
-	res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
-	close(start_pipefd[1]);
+	if (start_pipefd[1] != -1) {
+		st_stat = 0;
+		res = write(start_pipefd[1], pst_stat, sizeof(*pst_stat));
+		close(start_pipefd[1]);
+	}
+
+#ifdef WITH_SYSTEMD
+	if (flags & DAEMON_FLAGS_SYSTEMD_SERVICE)
+		sd_notify(1, "READY=1");
+#endif
 
 	state_mach_thid = pthread_self();
 	statemachine(NULL);
