@@ -82,65 +82,6 @@ static void mnts_cleanup(void *arg)
 	return;
 }
 
-/* When exiting direct mount triggers must be set catatonic, regardless
- * of whether they are busy on not, to avoid a hang on access once the
- * daemon has gone away.
- */
-static int set_direct_mount_catatonic(struct autofs_point *ap, struct mapent *me, int ioctlfd)
-{
-	struct ioctl_ops *ops = get_ioctl_ops();
-	unsigned int opened = 0;
-	char buf[MAX_ERR_BUF];
-	int fd = -1;
-	int error;
-
-	/* In case the miscellaneous device isn't being used try
-	 * and use an existing ioctl control fd. In this case if
-	 * we don't already have an ioctl fd the mount can't be
-	 * set catatonic if it's covered.
-	 */
-	if (ioctlfd >= 0)
-		fd = ioctlfd;
-	else if (me->ioctlfd >= 0)
-		fd = me->ioctlfd;
-	else {
-		error = ops->open(ap->logopt, &fd, me->dev, me->key);
-		if (error == -1) {
-			int err = errno;
-			char *estr;
-
-			estr = strerror_r(errno, buf, MAX_ERR_BUF);
-			error(ap->logopt,
-			      "failed to open ioctlfd for %s, error: %s",
-			      me->key, estr);
-			return err;
-		}
-		opened = 1;
-	}
-
-	if (fd >= 0) {
-		error = ops->catatonic(ap->logopt, fd);
-		if (error == -1) {
-			int err = errno;
-			char *estr;
-
-			estr = strerror_r(errno, buf, MAX_ERR_BUF);
-			error(ap->logopt,
-			      "failed to set %s catatonic, error: %s",
-			      me->key, estr);
-			if (opened)
-				ops->close(ap->logopt, fd);
-			return err;
-		}
-		if (opened)
-			ops->close(ap->logopt, fd);
-	}
-
-	debug(ap->logopt, "set %s catatonic", me->key);
-
-	return 0;
-}
-
 int do_umount_autofs_direct(struct autofs_point *ap, struct mnt_list *mnts, struct mapent *me)
 {
 	struct ioctl_ops *ops = get_ioctl_ops();
@@ -189,20 +130,16 @@ int do_umount_autofs_direct(struct autofs_point *ap, struct mnt_list *mnts, stru
 				error(ap->logopt,
 				      "ask umount returned busy for %s",
 				      me->key);
-				if (ap->state != ST_READMAP)
-					set_direct_mount_catatonic(ap, me, ioctlfd);
 				if (opened)
 					ops->close(ap->logopt, ioctlfd);
 				return 1;
 			} else {
 				me->ioctlfd = -1;
-				set_direct_mount_catatonic(ap, me, ioctlfd);
 				ops->close(ap->logopt, ioctlfd);
 				goto force_umount;
 			}
 		}
 		me->ioctlfd = -1;
-		set_direct_mount_catatonic(ap, me, ioctlfd);
 		ops->close(ap->logopt, ioctlfd);
 	} else {
 		error(ap->logopt,
@@ -232,8 +169,11 @@ int do_umount_autofs_direct(struct autofs_point *ap, struct mnt_list *mnts, stru
 			warn(ap->logopt, "mount point %s is in use", me->key);
 			if (ap->state == ST_SHUTDOWN_FORCE)
 				goto force_umount;
-			else
+			else {
+				if (ap->state != ST_READMAP)
+					set_direct_mount_tree_catatonic(ap, me);
 				return 0;
+			}
 			break;
 		case ENOTDIR:
 			error(ap->logopt, "mount point is not a directory");
@@ -297,12 +237,8 @@ int umount_autofs_direct(struct autofs_point *ap)
 			if (!error)
 				goto done;
 
-			error = set_direct_mount_catatonic(ap, me, me->ioctlfd);
-			if (!error)
-				goto done;
-
-			/* We really need to set this, last ditch attempt */
-			set_direct_mount_catatonic(ap, me, -1);
+			if (ap->state != ST_READMAP)
+				set_direct_mount_tree_catatonic(ap, me);
 done:
 			me = cache_enumerate(mc, me);
 		}
@@ -376,7 +312,7 @@ static int unlink_active_mounts(struct autofs_point *ap, struct mnt_list *mnts,
 
 	if (tree_get_mnt_list(mnts, &list, me->key, 1)) {
 		if (ap->state == ST_READMAP) {
-			time_t tout = me->source->exp_timeout;
+			time_t tout = get_exp_timeout(ap, me->source);
 			int save_ioctlfd, ioctlfd;
 
 			save_ioctlfd = ioctlfd = me->ioctlfd;
@@ -488,7 +424,7 @@ int do_mount_autofs_direct(struct autofs_point *ap,
 	}
 
 	/* In case the directory doesn't exist, try to mkdir it */
-	if (mkdir_path(me->key, 0555) < 0) {
+	if (mkdir_path(me->key, mp_mode) < 0) {
 		if (errno != EEXIST && errno != EROFS) {
 			crit(ap->logopt,
 			     "failed to create mount directory %s", me->key);
@@ -586,7 +522,7 @@ int mount_autofs_direct(struct autofs_point *ap)
 		}
 
 		mc = map->mc;
-		timeout = map->exp_timeout;
+		timeout = get_exp_timeout(ap, map);
 		cache_readlock(mc);
 		pthread_cleanup_push(cache_lock_cleanup, mc);
 		me = cache_enumerate(mc, NULL);
@@ -742,7 +678,7 @@ int mount_autofs_offset(struct autofs_point *ap, struct mapent *me, const char *
 	struct ioctl_ops *ops = get_ioctl_ops();
 	char buf[MAX_ERR_BUF];
 	struct mnt_params *mp;
-	time_t timeout = me->source->exp_timeout;
+	time_t timeout = get_exp_timeout(ap, me->source);
 	struct stat st;
 	int ioctlfd, status, ret;
 	const char *hosts_map_name = "-hosts";
@@ -803,7 +739,7 @@ int mount_autofs_offset(struct autofs_point *ap, struct mapent *me, const char *
 	strcat(mountpoint, offset);
 
 	/* In case the directory doesn't exist, try to mkdir it */
-	if (mkdir_path(mountpoint, 0555) < 0) {
+	if (mkdir_path(mountpoint, mp_mode) < 0) {
 		if (errno == EEXIST) {
 			/*
 			 * If the mount point directory is a real mount
@@ -905,7 +841,7 @@ void *expire_proc_direct(void *arg)
 	struct expire_args ec;
 	struct autofs_point *ap;
 	struct mapent *me = NULL;
-	unsigned int now;
+	unsigned int how;
 	int ioctlfd, cur_state;
 	int status, ret, left;
 
@@ -916,7 +852,7 @@ void *expire_proc_direct(void *arg)
 		fatal(status);
 
 	ap = ec.ap = ea->ap;
-	now = ea->when;
+	how = ea->how;
 	ec.status = -1;
 
 	ea->signaled = 1;
@@ -1010,7 +946,7 @@ void *expire_proc_direct(void *arg)
 
 			ioctlfd = me->ioctlfd;
 
-			ret = ops->expire(ap->logopt, ioctlfd, next->path, now);
+			ret = ops->expire(ap->logopt, ioctlfd, next->path, how);
 			if (ret) {
 				left++;
 				pthread_setcancelstate(cur_state, NULL);
@@ -1036,7 +972,7 @@ void *expire_proc_direct(void *arg)
 		debug(ap->logopt, "send expire to trigger %s", next->path);
 
 		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cur_state);
-		ret = ops->expire(ap->logopt, ioctlfd, next->path, now);
+		ret = ops->expire(ap->logopt, ioctlfd, next->path, how);
 		if (ret)
 			left++;
 		pthread_setcancelstate(cur_state, NULL);
@@ -1283,6 +1219,8 @@ static void *do_mount_direct(void *arg)
 
 	ap = mt.ap;
 
+	set_thread_mount_request_log_id(&mt);
+
 	args->signaled = 1;
 	status = pthread_cond_signal(&args->cond);
 	if (status)
@@ -1526,6 +1464,7 @@ int handle_packet_missing_direct(struct autofs_point *ap, autofs_packet_missing_
 	mt->type = NFY_MOUNT;
 	mt->uid = pkt->uid;
 	mt->gid = pkt->gid;
+	mt->pid = pkt->pid;
 	mt->wait_queue_token = pkt->wait_queue_token;
 
 	status = pthread_create(&thid, &th_attr_detached, do_mount_direct, mt);
