@@ -87,7 +87,7 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 {
 	const char *str_indirect = mount_type_str(t_indirect);
 	struct ioctl_ops *ops = get_ioctl_ops();
-	time_t timeout = ap->entry->maps->exp_timeout;
+	time_t timeout = get_exp_timeout(ap, ap->entry->maps);
 	char *options = NULL;
 	const char *hosts_map_name = "-hosts";
 	const char *map_name = hosts_map_name;
@@ -97,7 +97,12 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 	int ret;
 	int err;
 
-	ap->exp_runfreq = (timeout + CHECK_RATIO - 1) / CHECK_RATIO;
+	/* If the map is being shared the exp_timeout can't be inherited
+	 * from the map source since it may be different so the autofs
+	 * point exp_runfreq must have already been set.
+	 */
+	if (ap->entry->maps->ref <= 1)
+		ap->exp_runfreq = (timeout + CHECK_RATIO - 1) / CHECK_RATIO;
 
 	if (ops->version && !do_force_unlink) {
 		ap->flags |= MOUNT_FLAG_REMOUNT;
@@ -128,7 +133,7 @@ static int do_mount_autofs_indirect(struct autofs_point *ap, const char *root)
 	}
 
 	/* In case the directory doesn't exist, try to mkdir it */
-	if (mkdir_path(root, 0555) < 0) {
+	if (mkdir_path(root, mp_mode) < 0) {
 		if (errno != EEXIST && errno != EROFS) {
 			crit(ap->logopt,
 			     "failed to create autofs directory %s",
@@ -290,9 +295,6 @@ int umount_autofs_indirect(struct autofs_point *ap, const char *root)
 #endif
 	}
 
-	if (ap->shutdown)
-		ops->catatonic(ap->logopt, ap->ioctlfd);
-
 	ops->close(ap->logopt, ap->ioctlfd);
 	ap->ioctlfd = -1;
 	sched_yield();
@@ -390,7 +392,7 @@ void *expire_proc_indirect(void *arg)
 	struct mnt_list *mnts = NULL, *next;
 	struct expire_args *ea;
 	struct expire_args ec;
-	unsigned int now;
+	unsigned int how;
 	int offsets, submnts, count;
 	int retries;
 	int ioctlfd, cur_state;
@@ -403,7 +405,7 @@ void *expire_proc_indirect(void *arg)
 		fatal(status);
 
 	ap = ec.ap = ea->ap;
-	now = ea->when;
+	how = ea->how;
 	ec.status = -1;
 
 	ea->signaled = 1;
@@ -528,7 +530,7 @@ void *expire_proc_indirect(void *arg)
 		debug(ap->logopt, "expire %s", next->path);
 
 		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cur_state);
-		ret = ops->expire(ap->logopt, ioctlfd, next->path, now);
+		ret = ops->expire(ap->logopt, ioctlfd, next->path, how);
 		if (ret)
 			left++;
 		pthread_setcancelstate(cur_state, NULL);
@@ -542,7 +544,7 @@ void *expire_proc_indirect(void *arg)
 	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cur_state);
 	retries = (count_mounts(ap, ap->path, ap->dev) + 1);
 	while (retries--) {
-		ret = ops->expire(ap->logopt, ap->ioctlfd, ap->path, now);
+		ret = ops->expire(ap->logopt, ap->ioctlfd, ap->path, how);
 		if (ret)
 			left++;
 	}
@@ -733,6 +735,8 @@ static void *do_mount_indirect(void *arg)
 
 	ap = mt.ap;
 
+	set_thread_mount_request_log_id(&mt);
+
 	args->signaled = 1;
 	status = pthread_cond_signal(&args->cond);
 	if (status)
@@ -855,6 +859,7 @@ int handle_packet_missing_indirect(struct autofs_point *ap, autofs_packet_missin
 	mt->dev = pkt->dev;
 	mt->uid = pkt->uid;
 	mt->gid = pkt->gid;
+	mt->pid = pkt->pid;
 	mt->wait_queue_token = pkt->wait_queue_token;
 
 	status = pthread_create(&thid, &th_attr_detached, do_mount_indirect, mt);
