@@ -52,30 +52,6 @@ void st_mutex_unlock(void)
 		fatal(status);
 }
 
-void dump_state_queue(void)
-{
-	struct list_head *head = &state_queue;
-	struct list_head *p, *q;
-
-	logmsg("dumping queue");
-
-	list_for_each(p, head) {
-		struct state_queue *entry;
-
-		entry = list_entry(p, struct state_queue, list);
-		logmsg("queue list head path %s state %d busy %d",
-		      entry->ap->path, entry->state, entry->busy);
-
-		list_for_each(q, &entry->pending) {
-			struct state_queue *this;
-
-			this = list_entry(q, struct state_queue, pending);
-			logmsg("queue list entry path %s state %d busy %d",
-			      this->ap->path, this->state, this->busy);
-		}
-	}
-}
-
 void nextstate(int statefd, enum states next)
 {
 	char buf[MAX_ERR_BUF];
@@ -267,7 +243,7 @@ void expire_proc_cleanup(void *arg)
 	return;
 }
 
-static enum expire expire_proc(struct autofs_point *ap, int now)
+static enum expire expire_proc(struct autofs_point *ap, int how)
 {
 	pthread_t thid;
 	struct expire_args *ea;
@@ -295,7 +271,7 @@ static enum expire expire_proc(struct autofs_point *ap, int now)
 		fatal(status);
 
 	ea->ap = ap;
-	ea->when = now;
+	ea->how = how;
 	ea->status = 1;
 
 	if (ap->type == LKP_INDIRECT)
@@ -645,7 +621,7 @@ static unsigned int st_prepare_shutdown(struct autofs_point *ap)
 	ap->state = ST_SHUTDOWN_PENDING;
 
 	/* Unmount everything */
-	exp = expire_proc(ap, 1);
+	exp = expire_proc(ap, AUTOFS_EXP_IMMEDIATE);
 	switch (exp) {
 	case EXP_ERROR:
 	case EXP_PARTIAL:
@@ -671,7 +647,7 @@ static unsigned int st_force_shutdown(struct autofs_point *ap)
 	ap->state = ST_SHUTDOWN_FORCE;
 
 	/* Unmount everything */
-	exp = expire_proc(ap, 1);
+	exp = expire_proc(ap, AUTOFS_EXP_FORCE | AUTOFS_EXP_IMMEDIATE);
 	switch (exp) {
 	case EXP_ERROR:
 	case EXP_PARTIAL:
@@ -706,7 +682,7 @@ static unsigned int st_prune(struct autofs_point *ap)
 	assert(ap->state == ST_READY);
 	ap->state = ST_PRUNE;
 
-	switch (expire_proc(ap, 1)) {
+	switch (expire_proc(ap, AUTOFS_EXP_IMMEDIATE)) {
 	case EXP_ERROR:
 	case EXP_PARTIAL:
 		if (!ap->submount)
@@ -727,7 +703,7 @@ static unsigned int st_expire(struct autofs_point *ap)
 	assert(ap->state == ST_READY);
 	ap->state = ST_EXPIRE;
 
-	switch (expire_proc(ap, 0)) {
+	switch (expire_proc(ap, AUTOFS_EXP_NORMAL)) {
 	case EXP_ERROR:
 	case EXP_PARTIAL:
 		if (!ap->submount)
