@@ -8,6 +8,7 @@
 #ifndef AUTOMOUNT_H
 #define AUTOMOUNT_H
 
+#include <stdio.h>
 #include <paths.h>
 #include <limits.h>
 #include <time.h>
@@ -51,17 +52,10 @@
 #error Failed to locate umount(8)!
 #endif
 
-#ifndef HAVE_MODPROBE
-#error Failed to locate modprobe(8)!
-#endif
-
 #ifndef HAVE_LINUX_PROCFS
 #error Failed to verify existence of procfs filesystem!
 #endif
 
-#define FS_MODULE_NAME  "autofs4"
-int load_autofs4_module(void);
-
 /* The -s (sloppy) option to mount is good, if we have it... */
 
 #ifdef HAVE_SLOPPY_MOUNT
@@ -72,12 +66,21 @@ int load_autofs4_module(void);
 #define SLOPPY
 #endif
 
+#define DAEMON_FLAGS_FOREGROUND			0x0001
+#define DAEMON_FLAGS_SYSTEMD_SERVICE		0x0002
+#define DAEMON_FLAGS_HAVE_GLOBAL_OPTIONS	0x0004
+#define DAEMON_FLAGS_GHOST			0x0008
+#define DAEMON_FLAGS_CHECK_DAEMON		0x0010
+#define DAEMON_FLAGS_DUMP_MAPS			0x0020
+
 #define AUTOFS_SUPER_MAGIC 0x00000187L
 #define SMB_SUPER_MAGIC    0x0000517BL
 #define CIFS_MAGIC_NUMBER  0xFF534D42L
 #define NCP_SUPER_MAGIC    0x0000564CL
 #define NFS_SUPER_MAGIC    0x00006969L
 
+#define ATTEMPT_ID_SIZE 24
+
 /* This sould be enough for at least 20 host aliases */
 #define HOST_ENT_BUF_SIZE	2048
 
@@ -220,6 +223,7 @@ struct mapent *cache_partial_match(struct mapent_cache *mc, const char *prefix);
 struct mapent *cache_partial_match_wild(struct mapent_cache *mc, const char *prefix);
 int cache_add(struct mapent_cache *mc, struct map_source *ms, const char *key, const char *mapent, time_t age);
 int cache_update_offset(struct mapent_cache *mc, const char *mkey, const char *key, const char *mapent, time_t age);
+int cache_lookup_negative(struct mapent *me, const char *key);
 void cache_update_negative(struct mapent_cache *mc, struct map_source *ms, const char *key, time_t timeout);
 int cache_set_parents(struct mapent *mm);
 int cache_update(struct mapent_cache *mc, struct map_source *ms, const char *key, const char *mapent, time_t age);
@@ -243,6 +247,9 @@ const char **copy_argv(int argc, const char **argv);
 int compare_argv(int argc1, const char **argv1, int argc2, const char **argv2);
 int free_argv(int argc, const char **argv);
 
+struct pending_args;
+void set_thread_mount_request_log_id(struct pending_args *mt);
+
 void dump_core(void);
 int aquire_lock(void);
 void release_lock(void);
@@ -251,10 +258,19 @@ int spawnv(unsigned logopt, const char *prog, const char *const *argv);
 int spawn_mount(unsigned logopt, ...);
 int spawn_bind_mount(unsigned logopt, ...);
 int spawn_umount(unsigned logopt, ...);
+void open_mutex_lock(void);
+void open_mutex_unlock(void);
+int open_fd(const char *, int);
+int open_fd_mode(const char *, int, int);
+int open_pipe(int[2]);
+int open_sock(int, int, int);
+FILE *open_fopen_r(const char *);
+FILE *open_setmntent_r(const char *);
 void reset_signals(void);
 int do_mount(struct autofs_point *ap, const char *root, const char *name,
 	     int name_len, const char *what, const char *fstype,
 	     const char *options);
+extern unsigned int mp_mode;
 int mkdir_path(const char *path, mode_t mode);
 int rmdir_path(struct autofs_point *ap, const char *path, dev_t dev);
 
@@ -425,8 +441,9 @@ struct pending_args {
 	char name[PATH_MAX];		/* Name field of the request */
 	dev_t dev;			/* device number of mount */
 	unsigned int len;		/* Name field len */
-	uid_t uid;			/* uid of requestor */
-	gid_t gid;			/* gid of requestor */
+	uid_t uid;			/* uid of requester */
+	gid_t gid;			/* gid of requester */
+	pid_t pid;			/* pid of requestor */
 	unsigned long wait_queue_token;	/* Associated kernel wait token */
 };
 
@@ -502,12 +519,14 @@ struct thread_stdenv_vars {
 
 extern pthread_key_t key_thread_stdenv_vars;
 
+extern pthread_key_t key_thread_attempt_id;
+
 struct kernel_mod_version {
 	unsigned int major;
 	unsigned int minor;
 };
 
-/* Enable/disable gohsted directories */
+/* Enable/disable ghosted directories */
 #define MOUNT_FLAG_GHOST		0x0001
 
 /* Directory created for this mount? */
@@ -528,6 +547,13 @@ struct kernel_mod_version {
 /* Use symlinks instead of bind mounting local mounts */
 #define MOUNT_FLAG_SYMLINK		0x0040
 
+/* Read amd map even if it's not to be ghosted (browsable) */
+#define MOUNT_FLAG_AMD_CACHE_ALL	0x0080
+
+/* Set mount propagation for bind mounts */
+#define MOUNT_FLAG_SLAVE		0x0100
+#define MOUNT_FLAG_PRIVATE		0x0200
+
 struct autofs_point {
 	pthread_t thid;
 	char *path;			/* Mount point name */
@@ -540,6 +566,7 @@ struct autofs_point {
 	dev_t dev;			/* "Device" number assigned by kernel */
 	struct master_mapent *entry;	/* Master map entry for this mount */
 	unsigned int type;		/* Type of map direct or indirect */
+	time_t exp_timeout;		/* Indirect mount expire timeout */
 	time_t exp_runfreq;		/* Frequency for polling for timeouts */
 	time_t negative_timeout;	/* timeout in secs for failed mounts */
 	unsigned int flags;		/* autofs mount flags */
@@ -620,130 +647,5 @@ int alarm_start_handler(void);
 int alarm_add(struct autofs_point *ap, time_t seconds);
 void alarm_delete(struct autofs_point *ap);
 
-/*
- * Use CLOEXEC flag for open(), pipe(), fopen() (read-only case) and
- * socket() if possible.
- */
-static int cloexec_works;
-
-static inline void check_cloexec(int fd)
-{
-	if (cloexec_works == 0) {
-		int fl = fcntl(fd, F_GETFD);
-		if (fl != -1)
-			cloexec_works = (fl & FD_CLOEXEC) ? 1 : -1;
-	}
-	if (cloexec_works > 0)
-		return;
-	fcntl(fd, F_SETFD, FD_CLOEXEC);
-	return;
-}
-
-static inline int open_fd(const char *path, int flags)
-{
-	int fd;
-
-#if defined(O_CLOEXEC) && defined(SOCK_CLOEXEC)
-	if (cloexec_works != -1)
-		flags |= O_CLOEXEC;
-#endif
-	fd = open(path, flags);
-	if (fd == -1)
-		return -1;
-	check_cloexec(fd);
-	return fd;
-}
-
-static inline int open_fd_mode(const char *path, int flags, int mode)
-{
-	int fd;
-
-#if defined(O_CLOEXEC) && defined(SOCK_CLOEXEC)
-	if (cloexec_works != -1)
-		flags |= O_CLOEXEC;
-#endif
-	fd = open(path, flags, mode);
-	if (fd == -1)
-		return -1;
-	check_cloexec(fd);
-	return fd;
-}
-
-static inline int open_pipe(int pipefd[2])
-{
-	int ret;
-
-#if defined(O_CLOEXEC) && defined(SOCK_CLOEXEC) && defined(__have_pipe2)
-	if (cloexec_works != -1) {
-		ret = pipe2(pipefd, O_CLOEXEC);
-		if (ret != -1)
-			return 0;
-		if (errno != EINVAL)
-			return -1;
-	}
-#endif
-	ret = pipe(pipefd);
-	if (ret == -1)
-		return -1;
-	check_cloexec(pipefd[0]);
-	check_cloexec(pipefd[1]);
-	return 0;
-}
-
-static inline int open_sock(int domain, int type, int protocol)
-{
-	int fd;
-
-#ifdef SOCK_CLOEXEC
-	if (cloexec_works != -1)
-		type |= SOCK_CLOEXEC;
-#endif
-	fd = socket(domain, type, protocol);
-	if (fd == -1)
-		return -1;
-	check_cloexec(fd);
-	return fd;
-}
-
-static inline FILE *open_fopen_r(const char *path)
-{
-	FILE *f;
-
-#if defined(O_CLOEXEC) && defined(SOCK_CLOEXEC)
-	if (cloexec_works != -1) {
-		f = fopen(path, "re");
-		if (f != NULL) {
-			check_cloexec(fileno(f));
-			return f;
-		}
-	}
-#endif
-	f = fopen(path, "r");
-	if (f == NULL)
-		return NULL;
-	check_cloexec(fileno(f));
-	return f;
-}
-
-static inline FILE *open_setmntent_r(const char *table)
-{
-	FILE *tab;
-
-#if defined(O_CLOEXEC) && defined(SOCK_CLOEXEC)
-	if (cloexec_works != -1) {
-		tab = setmntent(table, "re");
-		if (tab != NULL) {
-			check_cloexec(fileno(tab));
-			return tab;
-		}
-	}
-#endif
-	tab = fopen(table, "r");
-	if (tab == NULL)
-		return NULL;
-	check_cloexec(fileno(tab));
-	return tab;
-}
-
 #endif
 
