@@ -32,6 +32,28 @@ static unsigned int logging_to_syslog = 0;
 static unsigned int do_verbose = 0;		/* Verbose feedback option */
 static unsigned int do_debug = 0;		/* Full debug output */
 
+static char *prepare_attempt_prefix(const char *msg)
+{
+	unsigned long *attempt_id;
+	char buffer[ATTEMPT_ID_SIZE + 1];
+	char *prefixed_msg = NULL;
+
+	attempt_id = pthread_getspecific(key_thread_attempt_id);
+	if (attempt_id) {
+		int len = sizeof(buffer) + 1 + strlen(msg) + 1;
+
+		snprintf(buffer, ATTEMPT_ID_SIZE, "%02lx", *attempt_id);
+		prefixed_msg = (char *) calloc(len, sizeof(char));
+		if (!prefixed_msg)
+			return NULL;
+		strcpy(prefixed_msg, buffer);
+		strcat(prefixed_msg, "|");
+		strcat(prefixed_msg, msg);
+	}
+
+	return prefixed_msg;
+}
+
 void set_log_norm(void)
 {
 	do_verbose = 0;
@@ -72,159 +94,223 @@ void set_log_debug_ap(struct autofs_point *ap)
 void log_info(unsigned int logopt, const char *msg, ...)
 {
 	unsigned int opt_log = logopt & (LOGOPT_DEBUG | LOGOPT_VERBOSE);
+	char *prefixed_msg;
 	va_list ap;
 
 	if (!do_debug && !do_verbose && !opt_log)
 		return;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_INFO, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_INFO, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
 
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void log_notice(unsigned int logopt, const char *msg, ...)
 {
 	unsigned int opt_log = logopt & (LOGOPT_DEBUG | LOGOPT_VERBOSE);
+	char *prefixed_msg;
 	va_list ap;
 
 	if (!do_debug && !do_verbose && !opt_log)
 		return;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_NOTICE, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_NOTICE, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
 
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void log_warn(unsigned int logopt, const char *msg, ...)
 {
 	unsigned int opt_log = logopt & (LOGOPT_DEBUG | LOGOPT_VERBOSE);
+	char *prefixed_msg;
 	va_list ap;
 
 	if (!do_debug && !do_verbose && !opt_log)
 		return;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_WARNING, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_WARNING, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
 
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void log_error(unsigned logopt, const char *msg, ...)
 {
+	char *prefixed_msg;
 	va_list ap;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_ERR, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_ERR, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
+
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void log_crit(unsigned logopt, const char *msg, ...)
 {
+	char *prefixed_msg;
 	va_list ap;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_CRIT, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_CRIT, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
+
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void log_debug(unsigned int logopt, const char *msg, ...)
 {
 	unsigned int opt_log = logopt & LOGOPT_DEBUG;
+	char *prefixed_msg;
 	va_list ap;
 
 	if (!do_debug && !opt_log)
 		return;
 
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_WARNING, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_WARNING, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
 
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
 void logmsg(const char *msg, ...)
 {
+	char *prefixed_msg;
 	va_list ap;
+
+	prefixed_msg = prepare_attempt_prefix(msg);
+
 	va_start(ap, msg);
-	if (logging_to_syslog)
-		vsyslog(LOG_CRIT, msg, ap);
-	else {
-		vfprintf(stderr, msg, ap);
+	if (logging_to_syslog) {
+		if (prefixed_msg)
+			vsyslog(LOG_CRIT, prefixed_msg, ap);
+		else
+			vsyslog(LOG_INFO, msg, ap);
+	} else {
+		if (prefixed_msg)
+			vfprintf(stderr, prefixed_msg, ap);
+		else
+			vfprintf(stderr, msg, ap);
 		fputc('\n', stderr);
 	}
 	va_end(ap);
+
+	if (prefixed_msg)
+		free(prefixed_msg);
+
 	return;
 }
 
-void log_to_syslog(void)
+void open_log(void)
 {
-	char buf[MAX_ERR_BUF];
-	int nullfd;
-
 	if (!syslog_open) {
 		syslog_open = 1;
 		openlog("automount", LOG_PID, LOG_DAEMON);
 	}
 
 	logging_to_syslog = 1;
-
-	/* Redirect all our file descriptors to /dev/null */
-	nullfd = open("/dev/null", O_RDWR);
-	if (nullfd < 0) {
-		char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
-		fprintf(stderr, "cannot open /dev/null: %s", estr);
-		exit(1);
-	}
-
-	if (dup2(nullfd, STDIN_FILENO) < 0 ||
-	    dup2(nullfd, STDOUT_FILENO) < 0 ||
-	    dup2(nullfd, STDERR_FILENO) < 0) {
-		char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
-		fprintf(stderr,
-			"redirecting file descriptors failed: %s", estr);
-		exit(1);
-	}
-
-	if (nullfd > 2)
-		close(nullfd);
-
 	return;
 }
 
