@@ -222,7 +222,7 @@ int check_nfs_mount_version(struct nfs_mount_vers *vers,
 	char *s_ver;
 	int cancel_state;
 
-	if (pipe(pipefd))
+	if (open_pipe(pipefd))
 		return -1;
 
 	pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &cancel_state);
@@ -230,6 +230,7 @@ int check_nfs_mount_version(struct nfs_mount_vers *vers,
 	sigfillset(&allsigs);
 	pthread_sigmask(SIG_BLOCK, &allsigs, &oldsig);
 
+	open_mutex_lock();
 	f = fork();
 	if (f == 0) {
 		reset_signals();
@@ -248,6 +249,7 @@ int check_nfs_mount_version(struct nfs_mount_vers *vers,
 
 	sigaddset(&tmpsig, SIGCHLD);
 	pthread_sigmask(SIG_SETMASK, &tmpsig, NULL);
+	open_mutex_unlock();
 
 	close(pipefd[1]);
 
@@ -487,8 +489,10 @@ void add_std_amd_vars(struct substvar *sv)
 		const struct substvar *v = macro_findvar(sv, "domain", 4);
 		if (v && *v->val) {
 			tmp = strdup(v->val);
-			if (tmp)
+			if (tmp) {
 				macro_global_addvar("cluster", 7, tmp);
+				free(tmp);
+			}
 		}
 	}
 
@@ -694,18 +698,9 @@ static struct ext_mount *ext_mount_lookup(const char *mountpoint)
 int ext_mount_add(struct list_head *entry, const char *path, unsigned int umount)
 {
 	struct ext_mount *em;
-	char *auto_dir;
 	u_int32_t hval;
 	int ret = 0;
 
-	/* Not a mount in the external mount directory */
-	auto_dir = conf_amd_get_auto_dir();
-	if (strncmp(path, auto_dir, strlen(auto_dir))) {
-		free(auto_dir);
-		return 0;
-	}
-	free(auto_dir);
-
 	pthread_mutex_lock(&ext_mount_hash_mutex);
 
 	em = ext_mount_lookup(path);
@@ -722,15 +717,12 @@ int ext_mount_add(struct list_head *entry, const char *path, unsigned int umount
 	}
 
 	em = malloc(sizeof(struct ext_mount));
-	if (!em) {
-		ret = -1;
+	if (!em)
 		goto done;
-	}
 
 	em->mountpoint = strdup(path);
 	if (!em->mountpoint) {
 		free(em);
-		ret = -1;
 		goto done;
 	}
 	em->umount = umount;
@@ -751,17 +743,8 @@ done:
 int ext_mount_remove(struct list_head *entry, const char *path)
 {
 	struct ext_mount *em;
-	char *auto_dir;
 	int ret = 0;
 
-	/* Not a mount in the external mount directory */
-	auto_dir = conf_amd_get_auto_dir();
-	if (strncmp(path, auto_dir, strlen(auto_dir))) {
-		free(auto_dir);
-		return 0;
-	}
-	free(auto_dir);
-
 	pthread_mutex_lock(&ext_mount_hash_mutex);
 
 	em = ext_mount_lookup(path);
@@ -786,6 +769,23 @@ done:
 	return ret;
 }
 
+int ext_mount_inuse(const char *path)
+{
+	struct ext_mount *em;
+	int ret = 0;
+
+	pthread_mutex_lock(&ext_mount_hash_mutex);
+	em = ext_mount_lookup(path);
+	if (!em)
+		goto done;
+
+	if (!list_empty(&em->mounts))
+		ret = 1;
+done:
+	pthread_mutex_unlock(&ext_mount_hash_mutex);
+	return ret;
+}
+
 /*
  * Get list of mounts under path in longest->shortest order
  */
@@ -1463,6 +1463,7 @@ void set_tsd_user_vars(unsigned int logopt, uid_t uid, gid_t gid)
 		error(logopt, "failed alloc tsv storage");
 		return;
 	}
+	memset(tsv, 0, sizeof(struct thread_stdenv_vars));
 
 	tsv->uid = uid;
 	tsv->gid = gid;
@@ -2028,14 +2029,16 @@ int umount_ent(struct autofs_point *ap, const char *path)
 {
 	int rv;
 
-	rv = spawn_umount(ap->logopt, path, NULL);
-	/* We are doing a forced shutcwdown down so unlink busy mounts */
-	if (rv && (ap->state == ST_SHUTDOWN_FORCE || ap->state == ST_SHUTDOWN)) {
-		if (ap->state == ST_SHUTDOWN_FORCE) {
-			info(ap->logopt, "forcing umount of %s", path);
-			rv = spawn_umount(ap->logopt, "-l", path, NULL);
-		}
+	if (ap->state != ST_SHUTDOWN_FORCE)
+		rv = spawn_umount(ap->logopt, path, NULL);
+	else {
+		/* We are doing a forced shutdown so unlink busy
+		 * mounts */
+		info(ap->logopt, "forcing umount of %s", path);
+		rv = spawn_umount(ap->logopt, "-l", path, NULL);
+	}
 
+	if (rv && (ap->state == ST_SHUTDOWN_FORCE || ap->state == ST_SHUTDOWN)) {
 		/*
 		 * Verify that we actually unmounted the thing.  This is a
 		 * belt and suspenders approach to not eating user data.
@@ -2058,6 +2061,63 @@ int umount_ent(struct autofs_point *ap, const char *path)
 	return rv;
 }
 
+int umount_amd_ext_mount(struct autofs_point *ap, struct amd_entry *entry)
+{
+	int rv = 1;
+
+	if (entry->umount) {
+		char *prog, *str;
+		char **argv;
+		int argc = -1;
+
+		str = strdup(entry->umount);
+		if (!str)
+			goto out;
+
+		prog = NULL;
+		argv = NULL;
+
+		argc = construct_argv(str, &prog, &argv);
+		if (argc == -1) {
+			free(str);
+			goto out;
+		}
+
+		if (!ext_mount_remove(&entry->ext_mount, entry->fs)) {
+			rv =0;
+			goto out_free;
+		}
+
+		rv = spawnv(ap->logopt, prog, (const char * const *) argv);
+		if (rv == -1 || (WIFEXITED(rv) && WEXITSTATUS(rv)))
+			error(ap->logopt,
+			     "failed to umount program mount at %s", entry->fs);
+		else {
+			rv = 0;
+			debug(ap->logopt,
+			      "umounted program mount at %s", entry->fs);
+			rmdir_path(ap, entry->fs, ap->dev);
+		}
+out_free:
+		free_argv(argc, (const char **) argv);
+		free(str);
+
+		goto out;
+	}
+
+	if (ext_mount_remove(&entry->ext_mount, entry->fs)) {
+		rv = umount_ent(ap, entry->fs);
+		if (rv)
+			error(ap->logopt,
+			      "failed to umount external mount %s", entry->fs);
+		else
+			debug(ap->logopt,
+			      "umounted external mount %s", entry->fs);
+	}
+out:
+	return rv;
+}
+
 static int do_mount_autofs_offset(struct autofs_point *ap,
 				  struct mapent *oe, const char *root,
 				  char *offset)
