@@ -48,11 +48,8 @@
 #include <stdint.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
-#include <arpa/inet.h>
-#include <net/if.h>
 #include <netinet/in.h>
 #include <netdb.h>
-#include <ifaddrs.h>
 
 #include "rpc_subs.h"
 #include "replicated.h"
@@ -62,34 +59,9 @@
 #define MAX_ERR_BUF		512
 #endif
 
-#define MAX_IFC_BUF		2048
-static int volatile ifc_buf_len = MAX_IFC_BUF;
-static int volatile ifc_last_len = 0;
-
-#define MASK_A  0x7F000000
-#define MASK_B  0xBFFF0000
-#define MASK_C  0xDFFFFF00
-
-/* Get numeric value of the n bits starting at position p */
-#define getbits(x, p, n)	((x >> (p + 1 - n)) & ~(~0 << n))
-
 #define mymax(x, y)	(x >= y ? x : y)
 #define mmax(x, y, z)	(mymax(x, y) == x ? mymax(x, z) : mymax(y, z))
 
-unsigned int ipv6_mask_cmp(uint32_t *host, uint32_t *iface, uint32_t *mask)
-{
-	unsigned int ret = 1;
-	unsigned int i;
-
-	for (i = 0; i < 4; i++) {
-		if ((host[i] & mask[i]) != (iface[i] & mask[i])) {
-			ret = 0;
-			break;
-		}
-	}
-	return ret;
-}
-
 void seed_random(void)
 {
 	int fd;
@@ -97,190 +69,21 @@ void seed_random(void)
 
 	fd = open_fd("/dev/urandom", O_RDONLY);
 	if (fd < 0) {
-		srandom(time(NULL));
+		srandom(monotonic_time(NULL));
 		return;
 	}
 
 	if (read(fd, &seed, sizeof(seed)) != -1)
 		srandom(seed);
 	else
-		srandom(time(NULL));
+		srandom(monotonic_time(NULL));
 
 	close(fd);
 
 	return;
 }
 
-static unsigned int get_proximity(struct sockaddr *host_addr)
-{
-	struct ifaddrs *ifa = NULL;
-	struct ifaddrs *this;
-	struct sockaddr_in *addr, *msk_addr, *if_addr;
-	struct sockaddr_in6 *addr6, *msk6_addr, *if6_addr;
-	struct in_addr *hst_addr;
-	struct in6_addr *hst6_addr;
-	int addr_len;
-	char buf[MAX_ERR_BUF];
-	uint32_t mask, ha, ia, *mask6, *ha6, *ia6;
-	int ret;
-
-	addr = NULL;
-	addr6 = NULL;
-	hst_addr = NULL;
-	hst6_addr = NULL;
-	mask6 = NULL;
-	ha6 = NULL;
-	ia6 = NULL;
-	ha = 0;
-
-	switch (host_addr->sa_family) {
-	case AF_INET:
-		addr = (struct sockaddr_in *) host_addr;
-		hst_addr = (struct in_addr *) &addr->sin_addr;
-		ha = ntohl((uint32_t) hst_addr->s_addr);
-		addr_len = sizeof(*hst_addr);
-		break;
-
-	case AF_INET6:
-#ifndef WITH_LIBTIRPC
-		return PROXIMITY_UNSUPPORTED;
-#else
-		addr6 = (struct sockaddr_in6 *) host_addr;
-		hst6_addr = (struct in6_addr *) &addr6->sin6_addr;
-		ha6 = &hst6_addr->s6_addr32[0];
-		addr_len = sizeof(*hst6_addr);
-		break;
-#endif
-
-	default:
-		return PROXIMITY_ERROR;
-	}
-
-	ret = getifaddrs(&ifa);
-	if (ret) {
-		char *estr = strerror_r(errno, buf, MAX_ERR_BUF);
-		logerr("getifaddrs: %s", estr);
-		return PROXIMITY_ERROR;
-	}
-
-	this = ifa;
-	while (this) {
-		if (!(this->ifa_flags & IFF_UP) ||
-		    this->ifa_flags & IFF_POINTOPOINT ||
-		    this->ifa_addr == NULL) {
-			this = this->ifa_next;
-			continue;
-		}
-
-		switch (this->ifa_addr->sa_family) {
-		case AF_INET:
-			if (host_addr->sa_family == AF_INET6)
-				break;
-			if_addr = (struct sockaddr_in *) this->ifa_addr;
-			ret = memcmp(&if_addr->sin_addr, hst_addr, addr_len);
-			if (!ret) {
-				freeifaddrs(ifa);
-				return PROXIMITY_LOCAL;
-			}
-			break;
-
-		case AF_INET6:
-#ifdef WITH_LIBTIRPC
-			if (host_addr->sa_family == AF_INET)
-				break;
-			if6_addr = (struct sockaddr_in6 *) this->ifa_addr;
-			ret = memcmp(&if6_addr->sin6_addr, hst6_addr, addr_len);
-			if (!ret) {
-				freeifaddrs(ifa);
-				return PROXIMITY_LOCAL;
-			}
-#endif
-		default:
-			break;
-		}
-		this = this->ifa_next;
-	}
-
-	this = ifa;
-	while (this) {
-		if (!(this->ifa_flags & IFF_UP) ||
-		    this->ifa_flags & IFF_POINTOPOINT ||
-		    this->ifa_addr == NULL) {
-			this = this->ifa_next;
-			continue;
-		}
-
-		switch (this->ifa_addr->sa_family) {
-		case AF_INET:
-			if (host_addr->sa_family == AF_INET6)
-				break;
-			if_addr = (struct sockaddr_in *) this->ifa_addr;
-			ia =  ntohl((uint32_t) if_addr->sin_addr.s_addr);
-
-			/* Is the address within a localy attached subnet */
-
-			msk_addr = (struct sockaddr_in *) this->ifa_netmask;
-			mask = ntohl((uint32_t) msk_addr->sin_addr.s_addr);
-
-			if ((ia & mask) == (ha & mask)) {
-				freeifaddrs(ifa);
-				return PROXIMITY_SUBNET;
-			}
-
-			/*
-			 * Is the address within a local ipv4 network.
-			 *
-			 * Bit position 31 == 0 => class A.
-			 * Bit position 30 == 0 => class B.
-			 * Bit position 29 == 0 => class C.
-			 */
-
-			if (!getbits(ia, 31, 1))
-				mask = MASK_A;
-			else if (!getbits(ia, 30, 1))
-				mask = MASK_B;
-			else if (!getbits(ia, 29, 1))
-				mask = MASK_C;
-			else
-				break;
-
-			if ((ia & mask) == (ha & mask)) {
-				freeifaddrs(ifa);
-				return PROXIMITY_NET;
-			}
-			break;
-
-		case AF_INET6:
-#ifdef WITH_LIBTIRPC
-			if (host_addr->sa_family == AF_INET)
-				break;
-			if6_addr = (struct sockaddr_in6 *) this->ifa_addr;
-			ia6 = &if6_addr->sin6_addr.s6_addr32[0];
-
-			/* Is the address within the network of the interface */
-
-			msk6_addr = (struct sockaddr_in6 *) this->ifa_netmask;
-			mask6 = &msk6_addr->sin6_addr.s6_addr32[0];
-
-			if (ipv6_mask_cmp(ha6, ia6, mask6)) {
-				freeifaddrs(ifa);
-				return PROXIMITY_SUBNET;
-			}
-
-			/* How do we define "local network" in ipv6? */
-#endif
-		default:
-			break;
-		}
-		this = this->ifa_next;
-	}
-
-	freeifaddrs(ifa);
-
-	return PROXIMITY_OTHER;
-}
-
-struct host *new_host(const char *name,
+struct host *new_host(const char *name, int ent_num,
 		      struct sockaddr *addr, size_t addr_len,
 		      unsigned int proximity, unsigned int weight,
 		      unsigned int options)
@@ -313,6 +116,7 @@ struct host *new_host(const char *name,
 	memset(new, 0, sizeof(struct host));
 
 	new->name = tmp1;
+	new->ent_num = ent_num;
 	new->addr_len = addr_len;
 	new->addr = tmp2;
 	new->proximity = proximity;
@@ -428,8 +232,7 @@ static unsigned int get_nfs_info(unsigned logopt, struct host *host,
 	socklen_t len = INET6_ADDRSTRLEN;
 	char buf[len + 1];
 	struct pmap parms;
-	struct timeval start, end;
-	struct timezone tz;
+	struct timespec start, end;
 	unsigned int supported = 0;
 	double taken = 0;
 	int status, count = 0;
@@ -464,20 +267,29 @@ static unsigned int get_nfs_info(unsigned logopt, struct host *host,
 				host->name, host->addr, host->addr_len,
 				proto, RPC_CLOSE_DEFAULT);
 		if (status == -EHOSTUNREACH) {
+			debug(logopt,
+			      "host not reachable getting portmap client");
 			supported = status;
 			goto done_ver;
-		} else if (status)
+		} else if (status) {
+			debug(logopt, "error 0x%d getting portmap client");
 			goto done_ver;
+		}
 		parms.pm_vers = NFS4_VERSION;
 		status = rpc_portmap_getport(pm_info, &parms, &rpc_info->port);
 		if (status == -EHOSTUNREACH || status == -ETIMEDOUT) {
+			debug(logopt,
+			      "host not reachable or timed out getting service port");
 			supported = status;
 			goto done_ver;
 		} else if (status < 0) {
 			if (version & NFS_VERS_MASK)
 				goto v3_ver; /* MOUNT_NFS_DEFAULT_PROTOCOL=4 */
-			else
+			else {
+				debug(logopt,
+				      "error 0x%d getting service port");
 				goto done_ver;
+			}
 		}
 	}
 
@@ -486,13 +298,15 @@ static unsigned int get_nfs_info(unsigned logopt, struct host *host,
 	else
 		status = rpc_tcp_getclient(rpc_info, NFS_PROGRAM, NFS4_VERSION);
 	if (status == -EHOSTUNREACH) {
+		debug(logopt, "host not reachable getting RPC client");
 		supported = status;
 		goto done_ver;
 	} else if (!status) {
-		gettimeofday(&start, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &start);
 		status = rpc_ping_proto(rpc_info);
-		gettimeofday(&end, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &end);
 		if (status == -ETIMEDOUT) {
+			debug(logopt, "host NFS ping timed out");
 			supported = status;
 			goto done_ver;
 		} else if (status > 0) {
@@ -503,7 +317,7 @@ static unsigned int get_nfs_info(unsigned logopt, struct host *host,
 				debug(logopt,
 				      "nfs v4 random selection time: %f", reply);
 			} else {
-				reply = elapsed(start, end);
+				reply = monotonic_elapsed(start, end);
 				debug(logopt, "nfs v4 rpc ping time: %f", reply);
 			}
 			taken += reply;
@@ -524,16 +338,23 @@ v3_ver:
 				host->name, host->addr, host->addr_len,
 				proto, RPC_CLOSE_DEFAULT);
 		if (status == -EHOSTUNREACH) {
+			debug(logopt,
+			      "host not reachable getting portmap client");
 			supported = status;
 			goto done_ver;
-		} else if (status)
+		} else if (status) {
+			debug(logopt,
+			      "error 0x%d getting getting portmap client");
 			goto done_ver;
+		}
 	}
 
 	if (!port) {
 		parms.pm_vers = NFS3_VERSION;
 		status = rpc_portmap_getport(pm_info, &parms, &rpc_info->port);
 		if (status == -EHOSTUNREACH || status == -ETIMEDOUT) {
+			debug(logopt,
+			      "host not reachable or timed out getting service port");
 			supported = status;
 			goto done_ver;
 		} else if (status < 0)
@@ -545,13 +366,15 @@ v3_ver:
 	else
 		status = rpc_tcp_getclient(rpc_info, NFS_PROGRAM, NFS3_VERSION);
 	if (status == -EHOSTUNREACH) {
+		debug(logopt, "host not reachable getting RPC client");
 		supported = status;
 		goto done_ver;
 	} else if (!status) {
-		gettimeofday(&start, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &start);
 		status = rpc_ping_proto(rpc_info);
-		gettimeofday(&end, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &end);
 		if (status == -ETIMEDOUT) {
+			debug(logopt, "host NFS ping timed out");
 			supported = status;
 			goto done_ver;
 		} else if (status > 0) {
@@ -562,7 +385,7 @@ v3_ver:
 				debug(logopt,
 				      "nfs v3 random selection time: %f", reply);
 			} else {
-				reply = elapsed(start, end);
+				reply = monotonic_elapsed(start, end);
 				debug(logopt, "nfs v3 rpc ping time: %f", reply);
 			}
 			taken += reply;
@@ -580,6 +403,8 @@ v2_ver:
 				host->name, host->addr, host->addr_len,
 				proto, RPC_CLOSE_DEFAULT);
 		if (status == -EHOSTUNREACH) {
+			debug(logopt,
+			      "host not reachable getting portmap client");
 			supported = status;
 			goto done_ver;
 		} else if (status)
@@ -590,6 +415,8 @@ v2_ver:
 		parms.pm_vers = NFS2_VERSION;
 		status = rpc_portmap_getport(pm_info, &parms, &rpc_info->port);
 		if (status == -EHOSTUNREACH || status == -ETIMEDOUT) {
+			debug(logopt,
+			      "host not reachable or timed out getting service port");
 			supported = status;
 			goto done_ver;
 		} else if (status < 0)
@@ -601,15 +428,17 @@ v2_ver:
 	else
 		status = rpc_tcp_getclient(rpc_info, NFS_PROGRAM, NFS2_VERSION);
 	if (status == -EHOSTUNREACH) {
+		debug(logopt, "host not reachable getting RPC client");
 		supported = status;
 		goto done_ver;
 	} else if (!status) {
-		gettimeofday(&start, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &start);
 		status = rpc_ping_proto(rpc_info);
-		gettimeofday(&end, &tz);
-		if (status == -ETIMEDOUT)
+		clock_gettime(CLOCK_MONOTONIC, &end);
+		if (status == -ETIMEDOUT) {
+			debug(logopt, "host NFS ping timed out");
 			supported = status;
-		else if (status > 0) {
+		} else if (status > 0) {
 			double reply;
 			if (random_selection) {
 				/* Random value between 0 and 1 */
@@ -617,7 +446,7 @@ v2_ver:
 				debug(logopt,
 				      "nfs v2 random selection time: %f", reply);
 			} else {
-				reply = elapsed(start, end);;
+				reply = monotonic_elapsed(start, end);;
 				debug(logopt, "nfs v2 rpc ping time: %f", reply);
 			}
 			taken += reply;
@@ -656,6 +485,29 @@ done_ver:
 	return supported;
 }
 
+static int check_address_proto(unsigned logopt,
+			       struct host *host, unsigned int version)
+{
+	int ipv6_requested = version & (TCP6_REQUESTED | UDP6_REQUESTED);
+	int ret = 1;
+
+	/* If a protocol has been explicitly requested then don't
+	 * consider addresses that don't match the requested protocol.
+	 */
+	if (ipv6_requested) {
+		if (host->addr_len == INET_ADDRSTRLEN)
+			ret = 0;
+	} else {
+		if (host->addr_len == INET6_ADDRSTRLEN)
+			ret = 0;
+	}
+
+	if (!ret)
+		debug(logopt, "requested protocol does not match address");
+
+	return ret;
+}
+
 static int get_vers_and_cost(unsigned logopt, struct host *host,
 			     unsigned int version, int port)
 {
@@ -664,6 +516,9 @@ static int get_vers_and_cost(unsigned logopt, struct host *host,
 	unsigned int supported, vers = (NFS_VERS_MASK | NFS4_VERS_MASK);
 	int ret = 0;
 
+	if (!check_address_proto(logopt, host, version))
+		return 0;
+
 	memset(&pm_info, 0, sizeof(struct conn_info));
 	memset(&rpc_info, 0, sizeof(struct conn_info));
 
@@ -720,8 +575,7 @@ static int get_supported_ver_and_cost(unsigned logopt, struct host *host,
 	struct conn_info pm_info, rpc_info;
 	int proto;
 	unsigned int vers;
-	struct timeval start, end;
-	struct timezone tz;
+	struct timespec start, end;
 	double taken = 0;
 	time_t timeout = RPC_TIMEOUT;
 	int status = 0;
@@ -734,6 +588,9 @@ static int get_supported_ver_and_cost(unsigned logopt, struct host *host,
 		debug(logopt, "called with host %s version 0x%x",
 			host->name, version);
 
+	if (!check_address_proto(logopt, host, version))
+		return 0;
+
 	memset(&pm_info, 0, sizeof(struct conn_info));
 	memset(&rpc_info, 0, sizeof(struct conn_info));
 
@@ -788,35 +645,41 @@ static int get_supported_ver_and_cost(unsigned logopt, struct host *host,
 		int ret = rpc_portmap_getclient(&pm_info,
 				host->name, host->addr, host->addr_len,
 				proto, RPC_CLOSE_DEFAULT);
-		if (ret)
+		if (ret) {
+			debug(logopt, "failed to get portmap client");
 			return 0;
+		}
 
 		memset(&parms, 0, sizeof(struct pmap));
 		parms.pm_prog = NFS_PROGRAM;
 		parms.pm_prot = rpc_info.proto;
 		parms.pm_vers = vers;
 		ret = rpc_portmap_getport(&pm_info, &parms, &rpc_info.port);
-		if (ret < 0)
+		if (ret < 0) {
+			debug(logopt, "failed to get service port");
 			goto done;
+		}
 	}
 
 	if (rpc_info.proto == IPPROTO_UDP)
 		status = rpc_udp_getclient(&rpc_info, NFS_PROGRAM, vers);
 	else
 		status = rpc_tcp_getclient(&rpc_info, NFS_PROGRAM, vers);
-	if (status == -EHOSTUNREACH)
+	if (status == -EHOSTUNREACH) {
+		status = 0;
+		debug(logopt, "host not reachable getting RPC client");
 		goto done;
-	else if (!status) {
-		gettimeofday(&start, &tz);
+	} else if (!status) {
+		clock_gettime(CLOCK_MONOTONIC, &start);
 		status = rpc_ping_proto(&rpc_info);
-		gettimeofday(&end, &tz);
+		clock_gettime(CLOCK_MONOTONIC, &end);
 		if (status > 0) {
 			if (random_selection) {
 				/* Random value between 0 and 1 */
 				taken = ((float) random())/((float) RAND_MAX+1);
 				debug(logopt, "random selection time %f", taken);
 			} else {
-				taken = elapsed(start, end);
+				taken = monotonic_elapsed(start, end);
 				debug(logopt, "rpc ping time %f", taken);
 			}
 		}
@@ -852,7 +715,7 @@ done:
 int prune_host_list(unsigned logopt, struct host **list,
 		    unsigned int vers, int port)
 {
-	struct host *this, *last, *first;
+	struct host *this, *last, *first, *prev;
 	struct host *new = NULL;
 	unsigned int proximity, selected_version = 0;
 	unsigned int v2_tcp_count, v3_tcp_count, v4_tcp_count;
@@ -1009,11 +872,18 @@ int prune_host_list(unsigned logopt, struct host **list,
 
 	first = last;
 	this = first;
+	prev = NULL;
 	while (this) {
 		struct host *next = this->next;
 		if (!this->name) {
 			remove_host(list, this);
 			add_host(&new, this);
+		} else if (defaults_use_hostname_for_mounts() && prev &&
+			   prev->ent_num == this->ent_num) {
+			/* When we use the hostname to mount, there is no
+			 * point in probing every address it has, just one is
+			 * enough.  Skip the rest.
+			 */
 		} else {
 			status = get_supported_ver_and_cost(logopt, this,
 						selected_version, port);
@@ -1021,6 +891,7 @@ int prune_host_list(unsigned logopt, struct host **list,
 				this->version = selected_version;
 				remove_host(list, this);
 				add_host(&new, this);
+				prev = this;
 			}
 		}
 		this = next;
@@ -1033,7 +904,7 @@ int prune_host_list(unsigned logopt, struct host **list,
 }
 
 static int add_new_host(struct host **list,
-			const char *host, unsigned int weight,
+			const char *host, int ent_num, unsigned int weight,
 			struct addrinfo *host_addr,
 			unsigned int rr, unsigned int options)
 {
@@ -1072,7 +943,7 @@ static int add_new_host(struct host **list,
 	else
 		return 0;
 
-	new = new_host(host, host_addr->ai_addr, addr_len, prx, weight, options);
+	new = new_host(host, ent_num, host_addr->ai_addr, addr_len, prx, weight, options);
 	if (!new)
 		return 0;
 
@@ -1085,7 +956,7 @@ static int add_new_host(struct host **list,
 	return 1;
 }
 
-static int add_host_addrs(struct host **list, const char *host,
+static int add_host_addrs(struct host **list, const char *host, int ent_num,
 			  unsigned int weight, unsigned int options)
 {
 	struct addrinfo hints, *ni, *this;
@@ -1110,7 +981,7 @@ static int add_host_addrs(struct host **list, const char *host,
 	}
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_flags = AI_NUMERICHOST;
+	hints.ai_flags = AI_NUMERICHOST | AI_CANONNAME;
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
 
@@ -1120,7 +991,7 @@ static int add_host_addrs(struct host **list, const char *host,
 
 	this = ni;
 	while (this) {
-		ret = add_new_host(list, host, weight, this, 0, options);
+		ret = add_new_host(list, host, ent_num, weight, this, 0, options);
 		if (!ret)
 			break;
 		this = this->ai_next;
@@ -1130,20 +1001,21 @@ static int add_host_addrs(struct host **list, const char *host,
 
 try_name:
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_flags = AI_ADDRCONFIG;
+	hints.ai_flags = AI_ADDRCONFIG | AI_CANONNAME;
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_DGRAM;
 
 	ret = getaddrinfo(name, NULL, &hints, &ni);
 	if (ret) {
-		error(LOGOPT_ANY, "hostname lookup failed: %s",
-		      gai_strerror(ret));
+		error(LOGOPT_ANY,
+		      "hostname lookup for %s failed: %s",
+		      name, gai_strerror(ret));
 		free(name);
 		return 0;
 	}
 
 	this = ni;
-	while (this->ai_next) {
+	while (this) {
 		if (this->ai_family == AF_INET) {
 			struct sockaddr_in *addr = (struct sockaddr_in *) this->ai_addr;
 			if (addr->sin_addr.s_addr != INADDR_LOOPBACK)
@@ -1159,7 +1031,7 @@ try_name:
 		rr++;
 	this = ni;
 	while (this) {
-		ret = add_new_host(list, host, weight, this, rr, options);
+		ret = add_new_host(list, host, ent_num, weight, this, rr, options);
 		if (!ret)
 			break;
 		this = this->ai_next;
@@ -1252,6 +1124,7 @@ int parse_location(unsigned logopt, struct host **hosts,
 {
 	char *str, *p, *delim;
 	unsigned int empty = 1;
+	int ent_num = 1;
 
 	if (!list)
 		return 0;
@@ -1309,7 +1182,7 @@ int parse_location(unsigned logopt, struct host **hosts,
 				}
 
 				if (p != delim) {
-					if (!add_host_addrs(hosts, p, weight, options)) {
+					if (!add_host_addrs(hosts, p, ent_num, weight, options)) {
 						if (empty) {
 							p = next;
 							continue;
@@ -1331,7 +1204,7 @@ int parse_location(unsigned logopt, struct host **hosts,
 				*delim = '\0';
 				next = delim + 1;
 
-				if (!add_host_addrs(hosts, p, weight, options)) {
+				if (!add_host_addrs(hosts, p, ent_num, weight, options)) {
 					p = next;
 					continue;
 				}
@@ -1345,6 +1218,7 @@ int parse_location(unsigned logopt, struct host **hosts,
 			return 0;
 		}
 
+		ent_num++;
 		p = next;
 	}
 
